<?php
/*
aviinfo

aviinfo -- Return information about an AVI file.

Description

Array aviinfo ( string filename)

Returns an associative array of information about <filename>.

The returned array has the following keys:

        duration        - length of clip in seconds
        v_scale                - time delay between frames in microseconds
        v_rate                - datarate (framerate = datarate / time scale)
        v_fps                - frames per second (datarate / time scale)
        v_frames        - total video frames
        v_width                - width of video stream in pixels
        v_height        - height of video stream in pixels
        v_ar                - aspect ratio of video stream (width/height) [:1]
        v_4cc                - FourCC code for video stream
        v_codec                - text string describing video codec - converted from 4cc - this                           data should be accurate, much was taken from Microsoft
        a_format        - WAVFORMATX format code for audio stream
        a_codec                - text string describing audio codec - converted from a_format -                           this data may be unreliable, taken from unknown C header
        a_brate                - average bitrate of audio stream in kilobits/second
        a_srate                - sample rate of audio stream in kilohertz
        a_chans                - number of audio channels
        a_mode                - text string describing audio stream (Stereo/Mono/[x]ch)

On error, aviinfo() will return '0' (not an Array). In the case of an error, the global
variable $aviinfo_error will include a text description of the last error.  You might want
to forgo this and comment out error_reporting(0) to allow php to report it's own errors.

*/
error_reporting(1);
/*
AVIInfo 1.0
Based on aviinfo.pm from openftpd
FOURCC codes and WAVFORMAT headers from various .h files and sites
Keenan Tims 2002
LGPL Licensing
*/
function aviinfo($filename) {
        function error($message) {
                global $aviinfo_error;
                $aviinfo_error = "$message";
        }
        $fourcc = array(                'ANIM' => "Intel - RDX",
                                                        'AUR2' => "AuraVision - Aura 2 Codec - YUV 422",
                                                        'AURA' => "AuraVision - Aura 1 Codec - YUV 411",
                                                        'BT20' => "Brooktree - MediaStream codec",
                                                        'BTCV' => "Brooktree - Composite Video codec",
                                                        'CC12' => "Intel - YUV12 codec",
                                                        'CDVC' => "Canopus - DV codec",
                                                        'CHAM' => "Winnov, Inc. - MM_WINNOV_CAVIARA_CHAMPAGNE",
                                                        'CPLA' => "Weitek - 4:2:0 YUV Planar",
                                                        'CVID' => "Supermac - Cinepak",
                                                        'CWLT' => "reserved",
                                                        'DUCK' => "Duck Corp. - TrueMotion 1.0",
                                                        'DVE2' => "InSoft - DVE-2 Videoconferencing codec",
                                                        'DXT1' => "reserved",
                                                        'DXT2' => "reserved",
                                                        'DXT3' => "reserved",
                                                        'DXT4' => "reserved",
                                                        'DXT5' => "reserved",
                                                        'DXTC' => "DirectX Texture Compression",
                                                        'FLJP' => "D-Vision - Field Encoded Motion JPEG With LSI Bitstream Format",
                                                        'GWLT' => "reserved",
                                                        'H260' => "Intel - Conferencing codec",
                                                        'H261' => "Intel - Conferencing codec",
                                                        'H262' => "Intel - Conferencing codec",
                                                        'H263' => "Intel - Conferencing codec",
                                                        'H264' => "Intel - Conferencing codec",
                                                        'H265' => "Intel - Conferencing codec",
                                                        'H266' => "Intel - Conferencing codec",
                                                        'H267' => "Intel - Conferencing codec",
                                                        'H268' => "Intel - Conferencing codec",
                                                        'H269' => "Intel - Conferencing codec",
                                                        'I263' => "Intel - I263",
                                                        'I420' => "Intel - Indeo 4 codec",
                                                        'IAN' => "Intel - RDX",
                                                        'ICLB' => "InSoft - CellB Videoconferencing codec",
                                                        'ILVC' => "Intel - Layered Video",
                                                        'ILVR' => "ITU-T - H.263+ compression standard",
                                                        'IRAW' => "Intel - YUV uncompressed",
                                                        'IV30' => "Intel - Indeo Video 3 codec",
                                                        'IV31' => "Intel - Indeo Video 3.1 codec",
                                                        'IV32' => "Intel - Indeo Video 3 codec",
                                                        'IV33' => "Intel - Indeo Video 3 codec",
                                                        'IV34' => "Intel - Indeo Video 3 codec",
                                                        'IV35' => "Intel - Indeo Video 3 codec",
                                                        'IV36' => "Intel - Indeo Video 3 codec",
                                                        'IV37' => "Intel - Indeo Video 3 codec",
                                                        'IV38' => "Intel - Indeo Video 3 codec",
                                                        'IV39' => "Intel - Indeo Video 3 codec",
                                                        'IV40' => "Intel - Indeo Video 4 codec",
                                                        'IV41' => "Intel - Indeo Video 4 codec",
                                                        'IV42' => "Intel - Indeo Video 4 codec",
                                                        'IV43' => "Intel - Indeo Video 4 codec",
                                                        'IV44' => "Intel - Indeo Video 4 codec",
                                                        'IV45' => "Intel - Indeo Video 4 codec",
                                                        'IV46' => "Intel - Indeo Video 4 codec",
                                                        'IV47' => "Intel - Indeo Video 4 codec",
                                                        'IV48' => "Intel - Indeo Video 4 codec",
                                                        'IV49' => "Intel - Indeo Video 4 codec",
                                                        'IV50' => "Intel - Indeo 5.0",
                                                        'MP42' => "Microsoft - MPEG-4 Video Codec V2",
                                                        'MPEG' => "Chromatic - MPEG 1 Video I Frame",
                                                        'MRCA' => "FAST Multimedia - Mrcodec",
                                                        'MRLE' => "Microsoft - Run Length Encoding",
                                                        'MSVC' => "Microsoft - Video 1",
                                                        'NTN1' => "Nogatech - Video Compression 1",
                                                        'qpeq' => "Q-Team - QPEG 1.1 Format video codec",
                                                        'RGBT' => "Computer Concepts - 32 bit support",
                                                        'RT21' => "Intel - Indeo 2.1 codec",
                                                        'RVX' => "Intel - RDX",
                                                        'SDCC' => "Sun Communications - Digital Camera Codec",
                                                        'SFMC' => "Crystal Net - SFM Codec",
                                                        'SMSC' => "Radius - proprietary",
                                                        'SMSD' => "Radius - proprietary",
                                                        'SPLC' => "Splash Studios - ACM audio codec",
                                                        'SQZ2' => "Microsoft - VXtreme Video Codec V2",
                                                        'SV10' => "Sorenson - Video R1",
                                                        'TLMS' => "TeraLogic - Motion Intraframe Codec",
                                                        'TLST' => "TeraLogic - Motion Intraframe Codec",
                                                        'TM20' => "Duck Corp. - TrueMotion 2.0",
                                                        'TMIC' => "TeraLogic - Motion Intraframe Codec",
                                                        'TMOT' => "Horizons Technology - TrueMotion Video Compression Algorithm",
                                                        'TR20' => "Duck Corp. - TrueMotion RT 2.0",
                                                        'V422' => "Vitec Multimedia - 24 bit YUV 4:2:2 format (CCIR 601)",
                                                        'V655' => "Vitec Multimedia - 16 bit YUV 4:2:2 format",
                                                        'VCR1' => "ATI - VCR 1.0",
                                                        'VIVO' => "Vivo - H.263 Video Codec",
                                                        'VIXL' => "Miro Computer Products AG",
                                                        'VLV1' => "Videologic - VLCAP.DRV",
                                                        'WBVC' => "Winbond Electronics - W9960",
                                                        'XLV0' => "NetXL, Inc. - XL Video Decoder",
                                                        'YC12' => "Intel - YUV12 codec",
                                                        'YUV8' => "Winnov, Inc. - MM_WINNOV_CAVIAR_YUV8",
                                                        'YUV9' => "Intel - YUV9",
                                                        'YUYV' => "Canopus - YUYV compressor",
                                                        'ZPEG' => "Metheus - Video Zipper",
                                                        'CYUV' => "Creative Labs, Inc - Creative Labs YUV",
                                                        'FVF1' => "Iterated Systems, Inc. - Fractal Video Frame",
                                                        'IF09' => "Intel - Intel Intermediate YUV9",
                                                        'JPEG' => "Microsoft - Still Image JPEG DIB",
                                                        'MJPG' => "Microsoft - Motion JPEG DIB Format",
                                                        'PHMO' => "IBM - Photomotion",
                                                        'ULTI' => "IBM - Ultimotion",
                                                        'VDCT' => "Vitec Multimedia - Video Maker Pro DIB",
                                                        'VIDS' => "Vitec Multimedia - YUV 4:2:2 CCIR 601 for V422",
                                                        'YU92' => "Intel - YUV",
                                                        '3IV1' => "3ivx",
                                                        '3IV2' => "3ivx",
                                                        'DIV3' => "DivX 3.11",
                                                        'DIV4' => "DivX",
                                                        'DIV5' => "DivX",
                                                        'DIVX' => "DivX 4+",
                                                        'divx' => "DivX 4+",
                                                        'DX50' => "DivX 5+",
                                                        'XVID' => "XviD"
                                                        );
        $wavformats = array (
                                                        '0' => "Unknown Format",
                                                        '1' => "PCM",
                                                        '2' => "Microsoft ADPCM",
                                                        '3' => "IEEE Float",
                                                        '4' => "Compaq Computer's VSELP",
                                                        '5' => "IBM CVSD",
                                                        '6' => "ALAW",
                                                        '7' => "MULAW",
                                                        '16' => "OKI ADPCM",
                                                        '17' => "Intel's DVI ADPCM",
                                                        '18' => "Videologic's MediaSpace ADPCM",
                                                        '19' => "Sierra ADPCM",
                                                        '20' => "G.723 ADPCM",
                                                        '21' => "DSP Solution's DIGISTD",
                                                        '22' => "DSP Solution's DIGIFIX",
                                                        '23' => "Dialogic OKI ADPCM",
                                                        '24' => "MediaVision ADPCM",
                                                        '25' => "HP CU",
                                                        '32' => "Yamaha ADPCM",
                                                        '33' => "Speech Compression's Sonarc",
                                                        '34' => "DSP Group's True Speech",
                                                        '35' => "Echo Speech's EchoSC1",
                                                        '36' => "Audiofile AF36",
                                                        '37' => "APTX",
                                                        '38' => "AudioFile AF10",
                                                        '39' => "Prosody 1612",
                                                        '40' => "LRC",
                                                        '48' => "Dolby AC2",
                                                        '49' => "GSM610",
                                                        '50' => "MSNAudio",
                                                        '51' => "Antex ADPCME",
                                                        '52' => "Control Res VQLPC",
                                                        '53' => "Digireal",
                                                        '54' => "DigiADPCM",
                                                        '55' => "Control Res CR10",
                                                        '56' => "NMS VBXADPCM",
                                                        '57' => "Roland RDAC",
                                                        '58' => "EchoSC3",
                                                        '59' => "Rockwell ADPCM",
                                                        '60' => "Rockwell Digit LK",
                                                        '61' => "Xebec",
                                                        '64' => "Antex Electronics G.721",
                                                        '65' => "G.728 CELP",
                                                        '66' => "MSG723",
                                                        '80' => "MPEG-1 Layer 1/2",
                                                        '81' => "RT24/PAC",
                                                        '85' => "MPEG-1 Layer 3",
                                                        '89' => "Cirrus",
                                                        '97' => "ESPCM",
                                                        '98' => "Voxware",
                                                        '99' => "Canopus Atrac",
                                                        '100' => "G.726 ADPCM",
                                                        '101' => "G.722 ADPCM",
                                                        '102' => "DSAT",
                                                        '103' => "DSAT Display",
                                                        '105' => "Voxware Byte Aligned",
                                                        '112' => "Voxware AC8",
                                                        '113' => "Voxware AC10",
                                                        '114' => "Voxware AC16",
                                                        '115' => "Voxware AC20",
                                                        '116' => "Voxware MetaVoice",
                                                        '117' => "Voxware MetaSound",
                                                        '118' => "Voxware RT29HW",
                                                        '119' => "Voxware VR12",
                                                        '120' => "Voxware VR18",
                                                        '121' => "Voxware TQ40",
                                                        '128' => "Softsound",
                                                        '129' => "Voxware TQ60",
                                                        '130' => "MSRT24",
                                                        '131' => "G.729A",
                                                        '132' => "MVI MV12",
                                                        '133' => "DF G.726",
                                                        '134' => "DF GSM610",
                                                        '136' => "ISIAudio",
                                                        '137' => "Onlive",
                                                        '145' => "SBC24",
                                                        '146' => "Dolby AC3 SPDIF",
                                                        '151' => "ZyXEL ADPCM",
                                                        '152' => "Philips LPCBB",
                                                        '153' => "Packed",
                                                        '256' => "Rhetorex ADPCM",
                                                        '257' => "BeCubed Software's IRAT",
                                                        '273' => "Vivo G.723",
                                                        '274' => "Vivo Siren",
                                                        '291' => "Digital G.723",
                                                        '512' => "Creative ADPCM",
                                                        '514' => "Creative FastSpeech8",
                                                        '515' => "Creative FastSpeech10",
                                                        '544' => "Quarterdeck",
                                                        '768' => "FM Towns Snd",
                                                        '1024' => "BTV Digital",
                                                        '1664' => "VME VMPCM",
                                                        '4096' => "OLIGSM",
                                                        '4097' => "OLIADPCM",
                                                        '4098' => "OLICELP",
                                                        '4099' => "OLISBC",
                                                        '4100' => "OLIOPR",
                                                        '4352' => "LH Codec",
                                                        '5120' => "Norris",
                                                        '5121' => "ISIAudio",
                                                        '5376' => "Soundspace Music Compression",
                                                        '8192' => "AC3",
                                                        '65534' => "SubFormat",
                                                        '65535' => "Development"
                                                         );


if (!$avi = fopen($filename,'r')) { error("Error opening file"); return(0); }
if (!$type = fread($avi,12)) { error("Error reading file"); return(0); }
if (substr($type,0,4) != 'RIFF') { error("No Valid RIFF header"); return(0); }
$header = NULL;
while (!$header) {
        if(!$data = fread($avi,8)) { error("Error reading file"); return(0); }
        if (substr($data,0,4) == 'LIST') {
                if(!$data = fread($avi,4)) { error("Error reading file"); return(0); }
                if (substr($data,0,4) == 'hdrl') {
                        if(!$header = fread($avi,10240)) { error("Error reading file"); return(0);}
                }
        }
        fclose($avi);
        $tag = 0;
        for ($i=0;$i < strlen($header);$i++) {
                $t = $i;
                $window = substr($header,$t,4);
                if ($window == 'strh') {
                        $t += 8;
                        $window = substr($header,$t,4);
                    if ($window == 'vids') {
                            $aviinfo['v_scale'] = (unpack("Vr",substr($header,$t+20,4)));
                            $aviinfo['v_rate'] = (unpack("Vr",substr($header,$t+24,4)));
                            $aviinfo['v_fps'] = $aviinfo['v_rate']['r'] / $aviinfo['v_scale']['r'];
                            $aviinfo['v_frames'] = (unpack("Vr",substr($header,$t+32,4)));
                            $aviinfo['duration'] = ($aviinfo['v_frames']['r'] / $aviinfo['v_fps']);
                            if ($aviinfo['v_fps'])
                                    $tag = 1;
                    } elseif ($window == 'auds') {
                            $tag = 2;
                    }
            } elseif ($window == 'strf') {
                        $t += 8;
                        $window = substr($header,$t,4);
                        if ($tag == 1) {
                            $aviinfo['v_width'] = (unpack("Vr",substr($header,$t+4,4)));
                            $aviinfo['v_height'] = (unpack("Vr",substr($header,$t+8,4)));
                            $aviinfo['v_ar'] = $aviinfo['v_width']['r'] / $aviinfo['v_height']['r'];
                            $aviinfo['v_codec'] = (substr($header,$t+16,4));
                        } elseif( $tag == 2 ) {
                            $aviinfo['a_codec'] = (unpack("vr",substr($header,$t,2)));
                            $aviinfo['a_chans'] = (unpack("vr",substr($header,$t+2,2)));
                            $a_brate = unpack("Vr",substr($header,$t+8,4));
                            $a_srate = unpack("Vr",substr($header,$t+4,4));
                            $aviinfo['a_brate'] = (8 * $a_brate['r'])/1000;
                            $aviinfo['a_srate'] = $a_srate['r']/1000;
                        }
                        $tag = 0;
            }
        }
}
        if ($wavformats[$aviinfo['a_codec']['r']])
                $a_codec = $wavformats[$aviinfo['a_codec']['r']];
        else
                $a_codec = "Unknown";
        switch ($aviinfo['a_chans']['r']) {
                case 1:
                        $a_mode = "Mono";
                        break;
                case 2:
                        $a_mode = "Stereo";
                        break;
                default:
                        $a_mode = $aviinfo['a_chans']['r']."ch";
                        break;
        }
        if ($fourcc[$aviinfo['v_codec']])
                $v_codec = $fourcc[$aviinfo['v_codec']];
    else
            $v_codec = "Unknown";

           $info = array ( 'v_scale' => $aviinfo['v_scale']['r'],
                                        'v_rate' => $aviinfo['v_rate']['r'],
                                        'v_fps' => round($aviinfo['v_fps'],3),
                                        'v_frames' => $aviinfo['v_frames']['r'],
                                        'duration' => round($aviinfo['duration'],0),
                                        'v_width' => $aviinfo['v_width']['r'],
                                        'v_height' => $aviinfo['v_height']['r'],
                                        'v_codec' => $v_codec,
                                        'v_4cc' => $aviinfo['v_codec'],
                                        'a_codec' => $a_codec,
                                        'a_format' => $aviinfo['a_codec']['r'],
                                        'a_chans' => $aviinfo['a_chans']['r'],
                                        'a_mode' => $a_mode,
                                        'a_brate' => round($aviinfo['a_brate'],1),
                                        'a_srate' => round($aviinfo['a_srate'],1),
                                        'v_ar' => round($aviinfo['v_ar'],2)
                                );
        return($info);
}

/*-----------------------------------------------------------------*/

?>
<SCRIPT LANGUAGE="javascript">
function openwin(url,name,w,h)
{
window.open(url,name,"width="+w+",height="+h+",toolbar=0,location=0,directories=0,menuBar=0,scrollbars=1,resizable=1,status=1,left=500,top=150");
}
</SCRIPT>

<?php
$t=$HTTP_GET_VARS["t"];
$f=$HTTP_GET_VARS["f"];

echo "<a href='../tools.php'>[Back]</a>";
echo "<form action='".$HTTP_SERVER_VARS["PHP_SELF"]."' method=get name=popup>";
echo "<b>$t/ </b><input type=text name=f value='$f'>";
echo " <input type=button name=pop value='...' onclick=\"openwin('../popup.php?var=f','popup','300','500')\">";
echo " <input type=submit name=sub value='View'>";
echo "<input type=hidden name=t value='$t'><hr>";
echo "</form>";

if(isset($f) )
{
 echo "File : ".$t."/".$f."<br>";
 $arr=aviinfo($t."/".$f);
 while ( list( $a, $b ) = each( $arr) )
 {
 echo  "<b>".$a." : </b>".$b."<br>" ;
 }
}
?>